; $Id: davegr1d__set.pro,v 1.7 2003/08/14 20:51:18 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr1D::set
; 
; PURPOSE:
;   Main set method for class. Used to set/reset class variables (all
;   class variables are 'private/protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     label: assign value (expects annotate structure) to object's label variable
;
;     inUse: assign value (expects integer) to object's inUse variable
;
;     dataObj: assign value (expects DaveDat* object) to object's dataObj variable.
;
;     lineType: assign the linetype index (integer)
;
;     lineSize: assign the line size (float)
;
;     symType: assign the symbol index (integer)
;
;     symSize: assign the symbol size (float)
;
;     lineSymFlag: assign the lineSymFlag which specifies whether
;     lines, symbols or symbols+lines are to be plotted 
;
;     myPlotType: assign the plot type (string): 'POINT' or
;     'HISTOGRAM'. Normally, the plot type is determined by the
;     independent axis type. However, it is possible to overide the
;     default type using this keyword in combination with the
;     myPlotTypeFlag keyword. 
;
;     myPlotTypeFlag: assign (integer) setting for myPlotTypeFlag
;     variable. If set to 1, then the myPlotType setting will be used
;     overiding the independent variable type. 
;
;     color: assign the line/symbol display color (int). Valid values are
;     defined by the custom color table created using "hfbs_getcolor.pro"
;
;     maskIndex: assign a list of independent axis (integer) indices to be masked
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveGr1D::set, lineType=lineType, lineSize=lineSize, symType=symType, symSize=symSize, $
                        lineSymFlag=lsFlag, color=color, maskIndex=maskIndex, position=pos, $
                        legend=legend,_EXTRA=etc

; reset !error_state structure, ie clear all errors
message,/reset

;use this to return a fail condition if at least one error is encountered
retVal = 1

; Set index 'pos' (from which changes should be made in the data
; array) to zero if not set
pos_IsSet = 0b
if (n_elements(pos) gt 0) then pos_IsSet = 1b else pos = 0

; nos of data elements
maxLen = self->count()

numericType=['FLOAT','INT','DOUBLE','LONG','BYTE']
integralType=['INT','LONG','BYTE']
floatType=['FLOAT','DOUBLE']
stringType=['STRING']

; lineType
len = n_elements(lineType)
if (len gt 0) then begin
    res = where((vartype(lineType) eq numericType), count) ; input must be a number 
    if (count ge 0) then begin
        len = len < maxLen
        self.lineType[pos:(pos+len)-1] = byte(lineType[0:len-1]); cast to byte
    endif else begin
        !error_state.msg = 'DaveGr1D::set: lineType keyword value should be an integer'
        !error_state.code = -1
        return, 0
    endelse
endif

; lineSize
len = n_elements(lineSize)
if (len gt 0) then begin
    res = where((vartype(lineSize) eq numericType), count) ; input must be a number 
    if (count ge 0) then begin
        len = len < maxLen
        self.lineSize[pos:(pos+len)-1] = float(lineSize[0:len-1]); cast to float
    endif else begin
        !error_state.msg = 'DaveGr1D::set: lineSize keyword value should be a float'
        !error_state.code = -1
        return, 0
    endelse
endif

; symType
len = n_elements(symtype)
if (len gt 0) then begin
    res = where((vartype(symType) eq numericType), count) ; input must be a number 
    if (count ge 0) then begin
        len = len < maxLen
        self.symType[pos:(pos+len)-1] = byte(symType[0:len-1]) ; cast to byte
    endif else begin
        !error_state.msg = 'DaveGr1D::set: symType keyword value should be an integer'
        !error_state.code = -1
        return, 0
    endelse
endif

; symSize
len = n_elements(symSize)
if (len gt 0) then begin
    res = where((vartype(symSize) eq numericType), count) ; input must be a number 
    if (count ge 0) then begin
        len = len < maxLen
        self.symSize[pos:(pos+len)-1] = byte(symSize[0:len-1]) ; cast to byte
    endif else begin
        !error_state.msg = 'DaveGr1D::set: symSize keyword value should be an integer'
        !error_state.code = -1
        return, 0
    endelse
endif

; lineSymFlag
len = n_elements(lsFlag)
if (len gt 0) then begin
    res = where((vartype(lsFlag) eq numericType), count) ; input must be a number 
    if (count ge 0) then begin
        len = len < maxLen
        self.lineSymFlag[pos:(pos+len)-1] = byte(lsFlag[0:len-1]) ; cast to byte
    endif else begin
        !error_state.msg = 'DaveGr1D::set: lineSymFlag keyword value should be an integer'
        !error_state.code = -1
        return, 0
    endelse
endif

; line/symbol Color; color should be 3-element byte array
len = n_elements(color)
if (len gt 0) then begin
    res = where((vartype(color) eq integralType), count) ; input must be a string 
    dim = size(color)
    len = (dim[0] eq 1)? 1 : dim[1]
    if ((count ge 0) and ((dim[0] eq 1) or (dim[0] eq 2)) and ((dim[1] eq 3) or (dim[2] eq 3))) then begin
        iend = (pos+len) < maxLen
        self.color[pos:iend-1,*] = (dim[0] eq 1)? color : color[0:len-1,*] 
    endif else begin
        !error_state.msg = 'DaveGr1D::set: color keyword value should be a byte array of [R,G,B] values'
        !error_state.code = -1
        return, 0
    endelse
endif

; maskIndex
if (n_elements(maskIndex) gt 0) then begin
    res = where((vartype(maskIndex) eq integralType), count) ; input must be an integer
    if (count ge 0) then begin
        if (ptr_valid(self.maskPtr)) then begin
            (*self.maskPtr) = fix(maskIndex) ; cast to integer
        endif else begin
            self.maskPtr = ptr_new(fix(maskIndex))
        endelse
    endif else begin
        !error_state.msg = 'DaveGr1D::set: maskIndex keyword value should be an integer vector'
        !error_state.code = -1
        return, 0
    endelse
endif

; set legend
if (n_elements(legend)) then begin
    ;; make sure value is a valid structure
    if(vartype(legend) eq 'LEGENDSTR') then self.legend = legend
endif

; Anything else is handled by the base class
retVal = self->DaveGr::set(_EXTRA=etc)

; However, if dataRef and addDataRef keywords were passed make sure that their values
; are valid DaveDat1D objects before passing resposibility to the base class
;if (n_elements(etc) gt 0) then begin
;    res = where(tag_names(etc) eq 'DATAREF', dataRef_isSet)
;    if (dataRef_isSet gt 0) then begin
;        if (vartype((etc.dataRef)[0]) ne 'DAVEDAT1D') then begin
;            !error_state.msg = 'DaveGr1D::set: invalid object reference (dataRef) passed'
;            !error_state.code = -1
;            return, 0
;        endif
;    endif 
;    res = where(tag_names(etc) eq 'ADDDATAREF', addDataRef_isSet)
;    if (addDataRef_isSet gt 0) then begin
;        if (vartype((etc.addDataRef)[0]) ne 'DAVEDAT1D') then begin
;            !error_state.msg = 'DaveGr1D::set: invalid object reference (dataRef) passed'
;            !error_state.code = -1
;            return, 0
;        endif
;    endif 
;    retVal = (pos_IsSet)? self->DaveGr::set(position=pos, _EXTRA=etc) : self->DaveGr::set(_EXTRA=etc)
;    ;retVal = self->DaveGr::set(_EXTRA=etc)
;endif

return, retVal
end
