; $Id: eqarrays.pro,v 1.3 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
;      eqarrays
;
; PURPOSE:
;
; 	Compares two arrays (vectors or matrices) of hopefully the same
; number of elements and returns true if every element of the two arrays 
; is the identical.
;
;	 The two arrays need not be structured the same. For example:	
;IDL> aa = [[1,2], [3, 4], [5, 6]]
;IDL> bb = [1, 2, 3, 4, 5, 6]
;IDL> print, eqarrays(aa, bb)
;   1
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED:
;
; EXAMPLE
;	IDL> dave_tagcontainer_example, 5
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;	TMR. Sept 8, 2003. Allow pointers to arrays. Added tolerance parameter.
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*
function eqArrays, $	; Returns 1 (true) if every element of the two arrays
			; is the same. Returns 0 otherwise.
	arrayA, $	; first array, any type. Or pointer to an array.
	arrayB, $ 	; second array, any type. Or pointer to an array.
	tol, $		; Optional tolerance factor. Only for numeric types.
	verbose = thisVerbose	; 
;/
; compares two arrays (vectors or matrices) of hopefully the same number of 
; elements and
; returns true if every element of the two arrays is the same.

  infolabel = "eqArrays >> '
  typeA = size(arrayA, /tname)
  typeB = size(arrayB, /tname)
  
  if (typeA NE 'POINTER') then begin
    lenA = n_elements(arrayA)
    ptr_a = ptr_new(arrayA)
  endif else begin
    lenA = n_elements(*arrayA)
    ptr_a = ptr_new(*arrayA)
  endelse

  if (typeB NE 'POINTER') then begin
    lenB = n_elements(arrayB)
    ptr_B = ptr_new(arrayB)
  endif else begin
    lenB = n_elements(*arrayB)
    ptr_B = ptr_new(*arrayB)

  endelse
  
  if (keyword_set(thisVerbose)) then begin
   if (lenA NE lenB) then begin
     print, infolabel, 'WARNING: Arrays are different lengths ', $
    	lenA, lenB
   endif
  endif
  
  if ((lenA EQ 0) or (lenB eq 0)) then begin
    return, 0
  endif
  nelems = min([lenA, lenB])

  
  if (arg_present(tol)) then  begin
    nNearlyZero = round(total( (abs(*ptr_a - *ptr_b) LE tol)))
   
  endif else begin
;    if (keyword_set(thisVerbose)) then begin
;      print, infolabel, "nelems=", nelems, $
;         " totalEq = ", total( (arrayA EQ arrayB)), $
;      	 " (arrayA EQ arrayB)=", (arrayA EQ arrayB)
;    endif
    nNearlyZero = round(total( *ptr_a EQ *ptr_b))
  endelse

   ptr_free, [ptr_a, ptr_b]
   return, (nNearlyZero EQ nelems)
  
end		;; eqArrays

;IDL> aa = [[1,2], [3, 4], [5, 6]]
;IDL> bb = [1, 2, 3, 4, 5, 6]
;IDL> print, (aa eq bb)
;   1   1
;   1   1
;   1   1
;IDL> print, ( bb eq aa)
;   1   1   1   1   1   1
;IDL> print, total( bb eq aa)
;      6.00000
;IDL> print, total(aa eq bb)
;      6.00000
